#define LCD_CS A3     // Chip Select goes to Analog 3
#define LCD_CD A2     // Command/Data goes to Analog 2
#define LCD_WR A1     // LCD Write goes to Analog 1
#define LCD_RD A0     // LCD Read goes to Analog 0
#define LCD_RESET A4  // Can alternately just connect to Arduino's reset pin

#include <SPI.h>           // f.k. for Arduino-1.5.2
#include "Adafruit_GFX.h"  // Hardware-specific library
//#include <Fonts/FreeMonoBoldOblique18pt7b.h>
#include <MCUFRIEND_kbv.h>
#include <TouchScreen.h>
MCUFRIEND_kbv tft;
uint8_t spi_save;

#define BLACK 0x0000
#define RED 0xF800
#define YELLOW 0xFFE0
#define WHITE 0xFFFF
#define GOLD 0xFEA0
#define GREY 0xC618

#define MINPRESSURE 200
#define MAXPRESSURE 1000

//3.5 Parameters
const int XP = 6, XM = A2, YP = A1, YM = 7;  //320x480 ID=0x9486
const int TS_LEFT = 915, TS_RT = 166, TS_TOP = 958, TS_BOT = 171;
TouchScreen ts = TouchScreen(XP, YP, XM, YM, 300);
TSPoint p;
bool down;
int pixel_x, pixel_y;

char currentPage;

#include <SD.h>
#define SD_CS 10  //5 // Card select for shield use
//OctBMP

void setup() {
  Serial.begin(9600);
  uint32_t when = millis();
  if (!Serial) delay(5000);  //allow some time for Leonardo
  Serial.println("Serial took " + String((millis() - when)) + "ms to start");
  //    tft.reset();                 //hardware reset
  uint16_t ID = tft.readID();  //
  Serial.print("ID = 0x");
  Serial.println(ID, HEX);
  if (ID == 0xD3D3) ID = 0x9481;  // write-only shield
                                  //    ID = 0x9329;                             // force ID
  tft.begin(ID);

  Serial.print(F("Initializing SD card..."));
  if (!SD.begin(SD_CS)) {
    Serial.println(F("failed!"));
    return;
  }
  Serial.println(F("OK!"));
  spi_save = SPCR;

  tft.setRotation(3);
  tft.fillScreen(BLACK);
  bmpDraw("Combined.bmp", 0, 0);
  currentPage = '0';
}

void loop() {
  if (currentPage == '0') {
    p = ts.getPoint();
    pinMode(YP, OUTPUT);  //restore shared pins
    pinMode(XM, OUTPUT);
    digitalWrite(YP, HIGH);
    digitalWrite(XM, HIGH);
    bool pressed = (p.z > MINPRESSURE && p.z < MAXPRESSURE);
    if (pressed) {
      pixel_x = map(p.y, TS_LEFT, TS_RT, 0, 480);
      pixel_y = map(p.x, TS_TOP, TS_BOT, 0, 320);
      if (pixel_x > 240 && pixel_y > 0 && pixel_y <280) {
        currentPage = '1';
        PensCalendar();
      } else if (pixel_x > 20 && pixel_x < 235 && pixel_y < 280) {
        currentPage = '2';
        tft.fillScreen(BLACK);
        bmpDraw("OSU.bmp", 0, 0);
      } else if (pixel_x > 0 && pixel_x < 40 && pixel_y > 280 && pixel_y < 320) {
        currentPage = '2';
        tft.fillScreen(BLACK);
      }
    }
  }

  if (currentPage == '1') {
    p = ts.getPoint();
    pinMode(YP, OUTPUT);  //restore shared pins
    pinMode(XM, OUTPUT);
    digitalWrite(YP, HIGH);
    digitalWrite(XM, HIGH);
    bool pressed = (p.z > MINPRESSURE && p.z < MAXPRESSURE);
    if (pressed) {
      pixel_x = map(p.y, TS_LEFT, TS_RT, 0, 480);
      pixel_y = map(p.x, TS_TOP, TS_BOT, 0, 320);
      if (pixel_x > 270 && pixel_x < 480 && pixel_y > 40 && pixel_y < 75) {
        currentPage = '2';
        tft.drawRect(30, 40, 180, 35, RED);
        tft.drawRect(29, 39, 182, 37, RED);
        delay(300);
        tft.fillScreen(BLACK);
        bmpDraw("Oct.bmp", 0, 0);
      } else if (pixel_x > 270 && pixel_x < 480 && pixel_y > 130 && pixel_y < 165) {
        currentPage = '2';
        tft.drawRect(30, 130, 180, 35, RED);
        tft.drawRect(29, 129, 182, 37, RED);
        delay(300);
        tft.fillScreen(BLACK);
        bmpDraw("Nov.bmp", 0, 0);
      } else if (pixel_x > 270 && pixel_x < 480 && pixel_y > 230 && pixel_y < 265) {
        currentPage = '2';
        tft.drawRect(30, 230, 180, 35, RED);
        tft.drawRect(29, 229, 182, 37, RED);
        delay(300);
        tft.fillScreen(BLACK);
        bmpDraw("Dec.bmp", 0, 0);
      } else if (pixel_x > 40 && pixel_x < 220 && pixel_y > 20 && pixel_y < 55) {
        currentPage = '2';
        tft.drawRect(255, 20, 180, 35, RED);
        tft.drawRect(254, 19, 182, 37, RED);
        delay(300);
        tft.fillScreen(BLACK);
        bmpDraw("Jan.bmp", 0, 0);
      } else if (pixel_x > 40 && pixel_x < 220 && pixel_y > 100 && pixel_y < 135) {
        currentPage = '2';
        tft.drawRect(255, 100, 180, 35, RED);
        tft.drawRect(254, 99, 182, 37, RED);
        delay(300);
        tft.fillScreen(BLACK);
        bmpDraw("Feb.bmp", 0, 0);
      } else if (pixel_x > 40 && pixel_x < 220 && pixel_y > 180 && pixel_y < 215) {
        currentPage = '2';
        tft.drawRect(255, 180, 180, 35, RED);
        tft.drawRect(254, 179, 182, 37, RED);
        delay(300);
        tft.fillScreen(BLACK);
        bmpDraw("Mar.bmp", 0, 0);
      } else if (pixel_x > 40 && pixel_x < 220 && pixel_y > 260 && pixel_y < 295) {
        currentPage = '2';
        tft.drawRect(255, 260, 180, 35, RED);
        tft.drawRect(254, 259, 182, 37, RED);
        delay(300);
        tft.fillScreen(BLACK);
        bmpDraw("Apr.bmp", 0, 0);
      }
    }
  }

  if (currentPage == '2') {
    p = ts.getPoint();
    pinMode(YP, OUTPUT);  //restore shared pins
    pinMode(XM, OUTPUT);
    digitalWrite(YP, HIGH);
    digitalWrite(XM, HIGH);
    bool pressed = (p.z > MINPRESSURE && p.z < MAXPRESSURE);
    if (pressed) {
      pixel_x = map(p.y, TS_LEFT, TS_RT, 0, 480);
      if (pixel_x > 0) {
        currentPage = '0';
        tft.fillScreen(BLACK);
        bmpDraw("Combined.bmp", 0, 0);
      }
    }
  }
}

#define BUFFPIXEL 20

void PensCalendar(void) {
  //tft.setFont(&FreeMonoBoldOblique18pt7b);
  tft.setTextSize(3);  //Uncomment if can get special font working
  tft.fillScreen(GOLD);
  tft.setCursor(35, 45);
  tft.setTextColor(BLACK);
  tft.println("October");
  tft.drawRect(30, 40, 180, 35, BLACK);
  tft.drawRect(29, 39, 182, 37, BLACK);


  tft.setCursor(260, 25);
  tft.println("January");
  tft.drawRect(255, 20, 180, 35, BLACK);
  tft.drawRect(254, 19, 182, 37, BLACK);

  tft.setCursor(35, 135);
  tft.println("November");
  tft.drawRect(30, 130, 180, 35, BLACK);
  tft.drawRect(29, 129, 182, 37, BLACK);

  tft.setCursor(35, 235);
  tft.println("December");
  tft.drawRect(30, 230, 180, 35, BLACK);
  tft.drawRect(29, 229, 182, 37, BLACK);

  tft.setCursor(260, 105);
  tft.println("February");
  tft.drawRect(255, 100, 180, 35, BLACK);
  tft.drawRect(254, 99, 182, 37, BLACK);

  tft.setCursor(260, 185);
  tft.println("March");
  tft.drawRect(255, 180, 180, 35, BLACK);
  tft.drawRect(254, 179, 182, 37, BLACK);

  tft.setCursor(260, 265);
  tft.println("April");
  tft.drawRect(255, 260, 180, 35, BLACK);
  tft.drawRect(254, 259, 182, 37, BLACK);
}

void bmpDraw(char *filename, int x, int y) {
  File bmpFile;
  int bmpWidth, bmpHeight;             // W+H in pixels
  uint8_t bmpDepth;                    // Bit depth (currently must be 24)
  uint32_t bmpImageoffset;             // Start of image data in file
  uint32_t rowSize;                    // Not always = bmpWidth; may have padding
  uint8_t sdbuffer[3 * BUFFPIXEL];     // pixel in buffer (R+G+B per pixel)
  uint16_t lcdbuffer[BUFFPIXEL];       // pixel out buffer (16-bit per pixel)
  uint8_t buffidx = sizeof(sdbuffer);  // Current position in sdbuffer
  boolean goodBmp = false;             // Set to true on valid header parse
  boolean flip = true;                 // BMP is stored bottom-to-top
  int w, h, row, col;
  uint8_t r, g, b;
  uint32_t pos = 0, startTime = millis();
  uint8_t lcdidx = 0;
  boolean first = true;

  if ((x >= tft.width()) || (y >= tft.height())) return;

  Serial.println();
  Serial.print("Loading image '");
  Serial.print(filename);
  Serial.println('\'');
  // Open requested file on SD card
  SPCR = spi_save;
  if ((bmpFile = SD.open(filename)) == NULL) {
    Serial.print("File not found");
    return;
  }

  // Parse BMP header
  if (read16(bmpFile) == 0x4D42) {  // BMP signature
    Serial.print(F("File size: "));
    Serial.println(read32(bmpFile));
    (void)read32(bmpFile);             // Read & ignore creator bytes
    bmpImageoffset = read32(bmpFile);  // Start of image data
    Serial.print(F("Image Offset: "));
    Serial.println(bmpImageoffset, DEC);
    // Read DIB header
    Serial.print(F("Header size: "));
    Serial.println(read32(bmpFile));
    bmpWidth = read32(bmpFile);
    bmpHeight = read32(bmpFile);
    if (read16(bmpFile) == 1) {    // # planes -- must be '1'
      bmpDepth = read16(bmpFile);  // bits per pixel
      Serial.print(F("Bit Depth: "));
      Serial.println(bmpDepth);
      if ((bmpDepth == 24) && (read32(bmpFile) == 0)) {  // 0 = uncompressed

        goodBmp = true;  // Supported BMP format -- proceed!
        Serial.print(F("Image size: "));
        Serial.print(bmpWidth);
        Serial.print('x');
        Serial.println(bmpHeight);

        // BMP rows are padded (if needed) to 4-byte boundary
        rowSize = (bmpWidth * 3 + 3) & ~3;

        // If bmpHeight is negative, image is in top-down order.
        // This is not canon but has been observed in the wild.
        if (bmpHeight < 0) {
          bmpHeight = -bmpHeight;
          flip = false;
        }

        // Crop area to be loaded
        w = bmpWidth;
        h = bmpHeight;
        if ((x + w - 1) >= tft.width()) w = tft.width() - x;
        if ((y + h - 1) >= tft.height()) h = tft.height() - y;

        // Set TFT address window to clipped image bounds
        SPCR = 0;
        tft.setAddrWindow(x, y, x + w - 1, y + h - 1);

        for (row = 0; row < h; row++) {  // For each scanline...
          // Seek to start of scan line.  It might seem labor-
          // intensive to be doing this on every line, but this
          // method covers a lot of gritty details like cropping
          // and scanline padding.  Also, the seek only takes
          // place if the file position actually needs to change
          // (avoids a lot of cluster math in SD library).
          if (flip)  // Bitmap is stored bottom-to-top order (normal BMP)
            pos = bmpImageoffset + (bmpHeight - 1 - row) * rowSize;
          else  // Bitmap is stored top-to-bottom
            pos = bmpImageoffset + row * rowSize;
          SPCR = spi_save;
          if (bmpFile.position() != pos) {  // Need seek?
            bmpFile.seek(pos);
            buffidx = sizeof(sdbuffer);  // Force buffer reload
          }

          for (col = 0; col < w; col++) {  // For each column...
            // Time to read more pixel data?
            if (buffidx >= sizeof(sdbuffer)) {  // Indeed
              // Push LCD buffer to the display first
              if (lcdidx > 0) {
                SPCR = 0;
                tft.pushColors(lcdbuffer, lcdidx, first);
                lcdidx = 0;
                first = false;
              }
              SPCR = spi_save;
              bmpFile.read(sdbuffer, sizeof(sdbuffer));
              buffidx = 0;  // Set index to beginning
            }

            // Convert pixel from BMP to TFT format
            b = sdbuffer[buffidx++];
            g = sdbuffer[buffidx++];
            r = sdbuffer[buffidx++];
            lcdbuffer[lcdidx++] = tft.color565(r, g, b);
          }  // end pixel
        }    // end scanline
        // Write any remaining data to LCD
        if (lcdidx > 0) {
          SPCR = 0;
          tft.pushColors(lcdbuffer, lcdidx, first);
        }
        Serial.print(F("Loaded in "));
        Serial.print(millis() - startTime);
        Serial.println(" ms");
      }  // end goodBmp
    }
  }

  bmpFile.close();
  if (!goodBmp) Serial.println("BMP format not recognized.");
}

uint16_t read16(File f) {
  uint16_t result;
  ((uint8_t *)&result)[0] = f.read();  // LSB
  ((uint8_t *)&result)[1] = f.read();  // MSB
  return result;
}

uint32_t read32(File f) {
  uint32_t result;
  ((uint8_t *)&result)[0] = f.read();  // LSB
  ((uint8_t *)&result)[1] = f.read();
  ((uint8_t *)&result)[2] = f.read();
  ((uint8_t *)&result)[3] = f.read();  // MSB
  return result;
}

/*bool Touch_getXY(void) {
  p = ts.getPoint();
  pinMode(YP, OUTPUT);  //restore shared pins
  pinMode(XM, OUTPUT);
  digitalWrite(YP, HIGH);
  digitalWrite(XM, HIGH);
  bool pressed = (p.z > MINPRESSURE && p.z < MAXPRESSURE);
  if (pressed) {
    if (tft.width() <= tft.height()) {                     //Portrait
      pixel_x = map(p.x, TS_LEFT, TS_RT, 0, tft.width());  //.kbv makes sense to me
      pixel_y = map(p.y, TS_TOP, TS_BOT, 0, tft.height());
    } else {
      pixel_x = map(p.y, TS_TOP, TS_BOT, 0, tft.width());
      pixel_y = map(p.x, TS_RT, TS_LEFT, 0, tft.height());
    }
  }
  return pressed;
}*/